﻿using System;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

namespace ELTE.TicTacToeGame
{
    /// <summary>
    /// Tic-Tac-Toe ablak típsua.
    /// </summary>
    public partial class TicTacToeForm : Form
    {
        #region Private fields

        private Button[,] gameBoard; // gombrács
        private Int32 stepNumber; // lépésszám
        private Char currentPlayer; // aktuális játékos karaktere

        #endregion

        #region Constructors

        /// <summary>
        /// Tic-Tac-Toe ablak létrehozása.
        /// </summary>
        public TicTacToeForm()
        {
            this.InitializeComponent();
        }

        #endregion

        #region Private methods

        /// <summary>
        /// Tábla létrehozása.
        /// </summary>
        private void GenerateBoard()
        {
            this.gameBoard = new Button[3, 3];
            for (Int32 i = 0; i < 3; i++)
                for (Int32 j = 0; j < 3; j++)
                {
                    this.gameBoard[i, j] = new Button();
                    this.gameBoard[i, j].Location = new Point(5 + 50 * i, 25 + 50 * j); // elhelyezkedés
                    this.gameBoard[i, j].Size = new Size(50, 50); // méret
                    this.gameBoard[i, j].Font = new Font(FontFamily.GenericSansSerif, Height / 10, FontStyle.Bold); // betűtípus
                    this.gameBoard[i, j].TabIndex = 100 + i * 3 + j; // a gomb számát a TabIndex-ben tároljuk
                    this.gameBoard[i, j].Dock = DockStyle.Fill; // kitölti a helyet
                    this.gameBoard[i, j].Click += new EventHandler(GameBoard_Click);
                    // közös eseménykezelő hozzárendelése minden gombhoz

                    this.tableLayoutPanel.Controls.Add(this.gameBoard[i, j]);
                    // felevesszük az ablakra a gombot
                }

            // létre kell hozni a megfelelő stípusokat a soroknak és oszlopoknak
            this.tableLayoutPanel.RowStyles.Clear();
            this.tableLayoutPanel.ColumnStyles.Clear();

            for (Int32 i = 0; i < 3; i++)
            {
                this.tableLayoutPanel.RowStyles.Add(new RowStyle(SizeType.Percent, 1 / 3F));
                // a sorok és oszlopok egyenlően, arányosan méreteződnek
            }
            for (Int32 j = 0; j < 3; j++)
            {
                this.tableLayoutPanel.ColumnStyles.Add(new ColumnStyle(SizeType.Percent, 1 / 3F));
            }

        }

        /// <summary>
        /// Új játék.
        /// </summary>
        private void NewGame()
        { 
            for (Int32 i = 0; i < 3; i++)
                for (Int32 j = 0; j < 3; j++)
                {
                    this.gameBoard[i, j].Text = String.Empty;
                    this.gameBoard[i, j].BackColor = Color.White;
                }

            this.currentPlayer = 'X';
            this.stepNumber = 0;
        }

        /// <summary>
        /// Játék ellenőrzése.
        /// </summary>
        private void CheckGame()
        {
            String won = String.Empty;

            for (int i = 0; i < 3; ++i) // ellenőrzések végrehajtása
            {
                if (this.gameBoard[i, 0].Text != String.Empty && this.gameBoard[i, 0].Text == this.gameBoard[i, 1].Text && this.gameBoard[i, 1].Text == this.gameBoard[i, 2].Text)
                    won = this.gameBoard[i, 0].Text;
            }
            for (int i = 0; i < 3; ++i)
            {
                if (this.gameBoard[0, i].Text != String.Empty && this.gameBoard[0, i].Text == this.gameBoard[1, i].Text && this.gameBoard[1, i].Text == this.gameBoard[2, i].Text)
                    won = this.gameBoard[0, i].Text;
            }
            if (this.gameBoard[0, 0].Text != String.Empty && this.gameBoard[0, 0].Text == this.gameBoard[1, 1].Text && this.gameBoard[1, 1].Text == this.gameBoard[2, 2].Text)
                won = this.gameBoard[0, 0].Text;
            if (this.gameBoard[0, 2].Text != String.Empty && this.gameBoard[0, 2].Text == this.gameBoard[1, 1].Text && this.gameBoard[1, 1].Text == this.gameBoard[2, 0].Text)
                won = this.gameBoard[0, 2].Text;

            switch (won)
            { 
                case "O":

                MessageBox.Show("A kör játékos győzött!", "Játék vége!", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    this.NewGame();
                    return;
                case "X":
                MessageBox.Show("A kereszt játékos győzött!", "Játék vége!", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    this.NewGame();
                    return;
            }
            
            if (this.stepNumber == 9) // döntetlen játék
            {
                MessageBox.Show("Döntetlen játék!", "Játék vége!", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                this.NewGame();
            }
        }

        /// <summary>
        /// Játék betöltése.
        /// </summary>
        /// <param name="fileName">A fájl elérési útvonala.</param>
        private void LoadGame(String fileName)
        {
            this.NewGame();

            using (StreamReader reader = new StreamReader(fileName)) // fájl megnyitása olvasásra
            {
                String[] characters = reader.ReadToEnd().Split(); // fájl tartalmának feldarabolása a whitespace karakterek mentén

                for (Int32 i = 0; i < 3; i++)
                    for (Int32 j = 0; j < 3; j++)
                    {
                        this.gameBoard[i, j].Text = characters[i * 3 + j]; // a következő karaktert beállítjuk címkének

                        // háttérszín és lépésszám beállítása
                        switch (this.gameBoard[i, j].Text)
                        {
                            case "O":
                                this.gameBoard[i, j].BackColor = Color.Yellow;
                                this.stepNumber++;
                                break;
                            case "X":
                                this.gameBoard[i, j].BackColor = Color.Orange;
                                this.stepNumber++;
                                break;
                        }
                    }

                // a következő játékos beállítása
                if (this.stepNumber % 2 == 0)
                    this.currentPlayer = 'X';
                else
                    this.currentPlayer = 'O';
            }
        }

        /// <summary>
        /// Játék mentése.
        /// </summary>
        /// <param name="fileName">A fájl elérési útvonala.</param>
        private void SaveGame(String fileName)
        {
            using (StreamWriter writer = new StreamWriter(fileName)) // fájl megnyitása írásra
            {
                for (Int32 i = 0; i < 3; i++)
                    for (Int32 j = 0; j < 3; j++)
                    {
                        writer.Write(this.gameBoard[i, j].Text + " "); // kiírjuk a feliratokat
                    }
            }
        }

        #endregion

        #region Form event handlers

        /// <summary>
        /// Ablak betöltésének eseménykezelése.
        /// </summary>
        private void TicTacToeForm_Load(object sender, EventArgs e)
        {
            this.GenerateBoard();
            this.NewGame();
        }

        /// <summary>
        /// Ablak méretváltásának eseménykezelése.
        /// </summary>
        private void TicTacToeForm_SizeChanged(object sender, EventArgs e)
        {
            for (Int32 i = 0; i < 3; i++)
                for (Int32 j = 0; j < 3; j++)
                {
                    // a gombok szövegméretét állítjuk be az új mérethez arányosan
                    this.gameBoard[i, j].Font = new Font(FontFamily.GenericSansSerif, Height / 10, FontStyle.Bold);
                }
        }

        #endregion

        #region Grid event handlers

        /// <summary>
        /// Gombrács eseménykezelője.
        /// </summary>
        private void GameBoard_Click(object sender, EventArgs e)
        {
            // a TabIndex-ből megkapjuk a sort és oszlopot
            Int32 x = ((sender as Button).TabIndex - 100) / 3;
            Int32 y = ((sender as Button).TabIndex - 100) % 3;

            this.gameBoard[x, y].BackColor = Color.Yellow;
            this.gameBoard[x, y].Text = this.currentPlayer.ToString();

            switch (this.currentPlayer)
            {
                case 'O':
                    this.gameBoard[x, y].BackColor = Color.Yellow;
                    this.currentPlayer = 'X';
                    break;
                case 'X':
                    this.gameBoard[x, y].BackColor = Color.Orange;
                    this.currentPlayer = 'O';
                    break;
            }

            this.stepNumber++;

            this.CheckGame();
        }

        #endregion

        #region Menu event handlers

        /// <summary>
        /// Új játék menüpontjának eseménykezelője.
        /// </summary>
        private void MenuGameNew_Click(object sender, EventArgs e)
        {
            this.NewGame();
        }

        /// <summary>
        /// Játék betöltés menüpontjának eseménykezelője.
        /// </summary>
        private void MenuGameLoad_Click(object sender, EventArgs e)
        {
            if (this.openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    this.LoadGame(openFileDialog.FileName);
                }
                catch
                {
                    MessageBox.Show("Hiba keletkezett a betöltés során.", "Tic-Tac-Toe", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    this.NewGame();
                }
            }
        }

        /// <summary>
        /// Játék mentés menüpontjának eseménykezelője.
        /// </summary>
        private void MenuGameSave_Click(object sender, EventArgs e)
        {
            if (this.saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    this.SaveGame(saveFileDialog.FileName);
                }
                catch
                {
                    MessageBox.Show("Hiba keletkezett a mentés során.", "Tic-Tac-Toe", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// Kilépés menüpontjának eseménykezelője.
        /// </summary>
        private void MenuGameExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        #endregion
    }
}
